/*
 * #%L
 * Maven helper plugin
 * 
 * $Author: tchemit $
 * $LastChangedDate: 2010-04-10 22:36:03 +0200 (sam., 10 avril 2010) $
 * $LastChangedRevision: 692 $
 * $Id: PropertiesDateRemoveFilterStream.java 692 2010-04-10 20:36:03Z tchemit $
 * $HeadURL: http://svn.nuiton.org/svn/maven-helper-plugin/tags/maven-helper-plugin-1.2.3/src/main/java/org/nuiton/io/PropertiesDateRemoveFilterStream.java $
 * %%
 * Copyright (C) 2009 - 2010 CodeLutin
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 * 
 * You should have received a copy of the GNU General Lesser Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * #L%
 */

package org.nuiton.io;

import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.OutputStream;

/**
 * Un ecrivain qui supprime la premiere ligne rencontree dans le flux.
 *
 *
 * <b>Note: </b> Attention, les performance d'utilisation de cet ecrivain
 * est problèmatique, car sur de gros fichiers (>1000 entrees) les
 * performances se degradent serieusement : pour 1200 entrees on arrive à
 * plus de 5 secondes, alors que sans on a 76 ms! ...
 *
 * FIXME : implanter quelque chose de plus performant dans tous les cas
 */
public class PropertiesDateRemoveFilterStream extends FilterOutputStream {

    private boolean firstLineOver;
    char endChar;

    public PropertiesDateRemoveFilterStream(OutputStream out) {
        super(out);
        firstLineOver = false;
        String lineSeparator = System.getProperty("line.separator");
        endChar = lineSeparator.charAt(lineSeparator.length() - 1);
    }

    @Override
    public void write(int b) throws IOException {
        if (!firstLineOver) {
            char c = (char) b;
            if (c == endChar) {
                firstLineOver = true;
            }
        } else {
            out.write(b);
        }
    }
}
